package org.vigia.client.communications;

import java.io.IOException;

import org.vigia.shared.communications.FrameListener;
import org.vigia.shared.communications.LogListener;

/**
 * The common interface for all the client communication implementations.
 * 
 * @author jrios
 */
public interface Communicator {
	
	/**
	 * Adds a listener to obtain the frames from the remote camera.
	 * 
	 * @param dataListener an implementation of the data listener.
	 */
	public void addFrameListener(FrameListener dataListener);
	
	/**
	 * Adds a listener to obtain control messages from the remote camera.
	 * 
	 * @param controlListener an implementation of the control listener.
	 */
	public void addLogListener(LogListener controlListener);
	
	/**
	 * Send to the remote camera a move command in an absolute way.
	 * 
	 * @param pan pan position.
	 * @param tilt tilt position.
	 * @param zoom zoom position.
	 * @throws IOException Error sending the command.
	 */
	public void move(int pan, int tilt, int zoom) throws IOException;
	
	/**
	 * Closes the communication with the camera.
	 * @throws IOException Error closing the communications.
	 */
	public void close() throws IOException;
	
	/**
	 * Check if the communications are closed.
	 * @return Is it's closed.
	 */
	public boolean isClosed();
}
