package org.vigia.client.communications;

import java.io.IOException;
import java.net.UnknownHostException;

import org.vigia.shared.communications.FrameListener;
import org.vigia.shared.communications.LogListener;

/**
 * An implementation of the client communication interface.
 * @author  jrios
 */
public class RemoteCameraCommunicator implements Communicator {
	
	/**
	 * The data communicator.
	 */
	protected DataCommunicator dataCommunicator;
	
	/**
	 * The control communicator.
	 */
	protected ControlCommunicator controlCommunicator;
	
	/**
	 * Creates a new communicator and connects with the remote server.
	 * 
	 * @param host The remote host.
	 * @param controlPort The remote control port.
	 * @param dataPort The local data port.
	 * @throws IOException Error with the IO.
	 * @throws UnknownHostException Host not found.
	 */
	public RemoteCameraCommunicator(String host, int controlPort, int dataPort) throws UnknownHostException, IOException {
		dataCommunicator = new DataCommunicator(dataPort);
		controlCommunicator = new ControlCommunicator(host, controlPort);
		controlCommunicator.init(dataPort);
	}
	
	/**
	 * @see org.vigia.client.communications.Communicator#addLogListener(org.vigia.shared.communications.LogListener)
	 */
	public void addLogListener(LogListener controlListener) {
		controlCommunicator.addLogListener(controlListener);
	}

	/**
	 * @see org.vigia.client.communications.Communicator#addFrameListener(org.vigia.shared.communications.FrameListener)
	 */
	public void addFrameListener(FrameListener dataListener) {
		dataCommunicator.addFrameListener(dataListener);
	}

	/**
	 * @see org.vigia.client.communications.Communicator#move(int, int, int)
	 */
	public void move(int pan, int tilt, int zoom) throws IOException {
		controlCommunicator.move(pan, tilt, zoom);
	}

	/**
	 * @see org.vigia.client.communications.Communicator#close()
	 */
	public void close() throws IOException {
		controlCommunicator.close();
		dataCommunicator.close();
	}

	/**
	 * @see org.vigia.client.communications.Communicator#isClosed()
	 */
	public boolean isClosed() {
		return controlCommunicator.isClosed();
	}
}
