
package org.vigia.client.gui.view;

import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;

/**
 * Dialog for specify ip, data port and control port
 * @author Daniel Hector Stolfi Rosso
 * @version 1.0 (25/11/08)
 */
@SuppressWarnings("serial")
public class AdvOptions extends JDialog {

	/**
	 * ip_panel: panel for IP address
	 */
	private Ip ip_panel;
	/**
	 * port_data_panel: panel for data port
	 */
	private Port port_data_panel;
	/**
	 * port_control_panel: panel for control port
	 */
	private Port port_control_panel;
	
	/**
	 * Constructor for Advanced Options Dialog
	 * @param owner JFrame Parent
	 * @param def_IP1 First byte of IP address
	 * @param def_IP2 Second byte of IP address
	 * @param def_IP3 Third byte of IP address
	 * @param def_IP4 Fourth byte of IP address
	 * @param def_data_port Data Port value
	 * @param def_control_port Control Port value
	 * @param ctr ActionListener in controller
	 * @param ActCmdOk Constant for action of button Ok
	 * @param ActCmdCancel Constant for action of button Cancel
	 */
	public AdvOptions(JFrame owner, int def_IP1, int def_IP2, int def_IP3, int def_IP4, int def_data_port, int def_control_port, ActionListener ctr, String ActCmdOk, String ActCmdCancel) {
		super(owner,"Opciones Avanzadas");
		setSize(270, 185);
		setLayout(null);
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setLocationRelativeTo(owner);
        setModal(true);
        setResizable(false);
       
        // Panel IP
        ip_panel = new Ip();
        add(ip_panel);
        ip_panel.setLocation(0, 10);
		ip_panel.setValues(def_IP1,def_IP2,def_IP3,def_IP4);
		        
		// Server Port
        port_data_panel = new Port("Puerto de Datos");
        add(port_data_panel);
        port_data_panel.setLocation(0,40);
		port_data_panel.setValue(def_data_port);        

		// Client Port
        port_control_panel = new Port("Puerto de Control");
        add(port_control_panel);
        port_control_panel.setLocation(0,70);
		port_control_panel.setValue(def_control_port);
        
		// Cancel Button
		JButton jB_Can = new JButton("Cancelar");
		jB_Can.addActionListener(ctr);
		jB_Can.setActionCommand(ActCmdCancel);		
		add(jB_Can);
		jB_Can.setBounds(150, 120, 105, 30);		

		// Connect Button 
		JButton jB_Ok = new JButton("OK");
		jB_Ok.addActionListener(ctr);
		jB_Ok.setActionCommand(ActCmdOk);
		add(jB_Ok);
		jB_Ok.setBounds(40, 120, 105, 30);        

	}
	
	/**
	 * Return the first IP byte
	 * @return The first IP byte
	 */
	public int getIP1() {
		return ip_panel.getIP1();
	}
	
	/**
	 * Return the second IP byte
	 * @return The second IP byte
	 */
	public int getIP2() {
		return ip_panel.getIP2();
	}

	/**
	 * Return the third IP byte
	 * @return The third IP byte
	 */
	public int getIP3() {
		return ip_panel.getIP3();
	}

	/**
	 * Return the fourth IP byte
	 * @return The fourth IP byte
	 */
	public int getIP4() {
		return ip_panel.getIP4();
	}

	/**
	 * Return the server port
	 * @return The server port
	 */
	public int getDataPort() {
		return port_data_panel.getPort();
	}
	
	/**
	 * Return the client port
	 * @return The client port
	 */
	public int getControlPort() {
		return port_control_panel.getPort();
	}

}
