
package org.vigia.client.gui.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import org.vigia.client.exception.VigiaClientException;
import org.vigia.client.gui.controller.Controller;
import org.vigia.shared.communications.ELog;


/**
 * GUI for client
 */
@SuppressWarnings("serial")
public class Panel extends JFrame implements View {

	/**
	 * ip1: First Integer of IP address
	 */
	private int ip1;
	/**
	 * ip2: Second Integer of IP address
	 */
	private int ip2;
	/**
	 * ip3: Third Integer of IP address
	 */
	private int ip3;
	/**
	 * ip4: Fourth Integer of IP address
	 */
	private int ip4;
	/**
	 * data_port: Integer for data port
	 */
	private int data_port;
	/**
	 * control_port: Integer for control port
	 */
	private int control_port;
	/**
	 * target_dir: Directory for screen shot 
	 */
	private String target_dir;
	/**
	 * connectVisible: State of connect panel
	 */
	private boolean connectVisible;
	/**
	 * advOptionsVisible: State of advOptions panel 
	 */
	private boolean advOptionsVisible;
	/**
	 * jMenuBar: Application's menu bar
	 */
    private JMenuBar jMenuBar;
    /**
     * jMenu_File: Menu file
     */
    private JMenu jMenu_File;
    /**
     * jMenu_Tools: Menu tools
     */
    private JMenu jMenu_Tools;
    /**
     * jMenu_Help: Menu help
     */
    private JMenu jMenu_Help;
    /**
     * jMI_Conn: Menu item Connect
     */
    private JMenuItem jMI_Conn;
    /**
     * jMI_Disc: Menu item Disconnect
     */
    private JMenuItem jMI_Disc;
    /**
     * jMI_Close: Menu item Close
     */
    private JMenuItem jMI_Close;
    /**
     * jMI_Config: Menu item Config
     */
    private JMenuItem jMI_Config;
    /**
     * jMI_ScrShot: Menu item Screen Shot
     */
    private JMenuItem jMI_ScrShot;
    /**
     * jMI_ShowLog: Menu item Show Log
     */
    private JMenuItem jMI_ShowLog;
    /**
     * jMI_About: Menu item About
     */
    private JMenuItem jMI_About;
    /**
     * imPanel: Image's Panel
     */
    private ImagePanel imPanel;
    /**
     * dialogConnect: Dialog for Connect
     */
    private Connect dialogConnect;
    /**
     * dialogAdvOptions: Dialog for Advanced Options
     */
    private AdvOptions dialogAdvOptions;
    /**
     * dialogScreenShot: Dialog for Screen Shot
     */
    private ScreenShot dialogScreenShot;
    /**
     * al: Action Listener in controller
     */
    private ActionListener al;
    /**
     * logDetails: Frame for log details
     */
    private Log logDetails=null;
    /**
     * jL_StatusBar: Label for status bar
     */
    private JLabel jL_StatusBar;

	/**
	 * Constructor for the Panel
	 * @param title Title of JFrame
	 * @param def_ip Default IP
	 * @param def_data_port Default data port
	 * @param def_control_port Default control port
	 * @param def_log_file Default log file
	 * @param def_scrshot_dir Default screen shot path
	 */
	public Panel(String title, String def_ip, String def_data_port, String def_control_port, String def_log_file, String def_scrshot_dir) {
		super(title);
		setSize(800, 600); // If the size of image is 640x480
		setLocationRelativeTo(null);
		//setLayout(null);
		setLayout(new BorderLayout());
        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setResizable(false);
        
        // suppose right data 
        StringTokenizer st = new StringTokenizer(def_ip,".");
        ip1 = Integer.parseInt(st.nextToken());
        ip2 = Integer.parseInt(st.nextToken());
        ip3 = Integer.parseInt(st.nextToken());
        ip4 = Integer.parseInt(st.nextToken());             
        data_port = Integer.valueOf(def_data_port);
        control_port = Integer.valueOf(def_control_port);
advOptionsVisible = false;
        connectVisible = false;
        target_dir = def_scrshot_dir;
        
        // Create the menu bar.
        jMenuBar = new JMenuBar();

        // File menu 
        jMenu_File = new JMenu("Archivo");
        jMenu_File.setMnemonic(KeyEvent.VK_A);
        jMenuBar.add(jMenu_File);

        // JMenuItems for menu File
        jMI_Conn = new JMenuItem("Conectar",KeyEvent.VK_C);
        jMenu_File.add(jMI_Conn);
        jMI_Disc = new JMenuItem("Desconectar",KeyEvent.VK_D);
        jMenu_File.add(jMI_Disc);
        jMI_Close = new JMenuItem("Cerrar",KeyEvent.VK_R);
        jMenu_File.add(jMI_Close);

        // Tools menu
        jMenu_Tools = new JMenu("Herramientas");
        jMenu_Tools.setMnemonic(KeyEvent.VK_H);
        jMenuBar.add(jMenu_Tools);

        // JMenuItems for menu Tools
        jMI_Config = new JMenuItem("Configuracin de la conexin",KeyEvent.VK_C);
        jMenu_Tools.add(jMI_Config);
        jMI_ScrShot = new JMenuItem("Captura de pantalla",KeyEvent.VK_P);
        jMenu_Tools.add(jMI_ScrShot);
        jMI_ShowLog = new JMenuItem("Mostrar log",KeyEvent.VK_L);
        jMenu_Tools.add(jMI_ShowLog);
        
        // Help menu
        jMenu_Help = new JMenu("Ayuda");
        jMenu_Help.setMnemonic(KeyEvent.VK_Y);
        jMenuBar.add(jMenu_Help);

        // JMenuItems for menu Help
        jMI_About = new JMenuItem("Acerca de...",KeyEvent.VK_A);
        jMenu_Help.add(jMI_About);        
        jMenuBar.add(jMenu_Help);
        
        // adds MenuBar to Frame
        setJMenuBar(jMenuBar);
        
        // adds Image
        imPanel = new ImagePanel(true); // Doble Buffered
        imPanel.setPreferredSize(new Dimension(640,480));
        add(imPanel,BorderLayout.CENTER);
        
        // Status Bar
        jL_StatusBar = new JLabel();
        jL_StatusBar.setPreferredSize(new Dimension(800,25));
        add(jL_StatusBar,java.awt.BorderLayout.SOUTH);
        jL_StatusBar.setBorder(BorderFactory.createRaisedBevelBorder());
        
        // creates Log
        logDetails = new Log("Fichero de Registro (LOG)");
        
        setVisible(true);	
	}    
        
    /**
	 * @see org.vigia.client.gui.view.View#getControlPort()
	 */
	public String getControlPort() throws VigiaClientException {
		if (advOptionsVisible) control_port = dialogAdvOptions.getControlPort();		
		return new Integer(control_port).toString();
	}

	/**
	 * @see org.vigia.client.gui.view.View#getDataPort()
	 */
	public String getDataPort() throws VigiaClientException {
		if (advOptionsVisible) data_port = dialogAdvOptions.getDataPort();		
		return new Integer(data_port).toString();
	}

	/**
	 * @see org.vigia.client.gui.view.View#getIPServer()
	 */
	public String getIPServer() throws VigiaClientException {
		if (advOptionsVisible) {
			ip1 = dialogAdvOptions.getIP1();
			ip2 = dialogAdvOptions.getIP2();
			ip3 = dialogAdvOptions.getIP3();
			ip4 = dialogAdvOptions.getIP4();
			if (connectVisible) dialogConnect.setIP(ip1, ip2, ip3, ip4); 
		} else {
			ip1 = dialogConnect.getIP1();
			ip2 = dialogConnect.getIP2();
			ip3 = dialogConnect.getIP3();
			ip4 = dialogConnect.getIP4();			
		}
		return ip1+"."+ip2+"."+ip3+"."+ip4;
	}

	/**
	 * @see org.vigia.client.gui.view.View#getImage()
	 */
	public BufferedImage getImage() throws VigiaClientException {
		return imPanel.getImage();
	}

	/**
	 * @see org.vigia.client.gui.view.View#getResolution()
	 */
	public String getResolution() throws VigiaClientException {
	    // Get the size of the default screen
	    Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();	    
		return new Integer(dim.width).toString()+'x'+new Integer(dim.height).toString();
	}

	/**
	 * @see org.vigia.client.gui.view.View#getScreenShotURL()
	 */
	public String getScreenShotURL() throws VigiaClientException {
		target_dir = dialogScreenShot.getSelectedDir();
		return target_dir;
	}

	/**
	 * @see org.vigia.client.gui.view.View#showAboutInfo()
	 */
	public boolean showAboutInfo() throws VigiaClientException {
		// TODO Add Icon and complete text
		JOptionPane.showMessageDialog(this,
									  "VIGIA 1.0",
									  "Acerca de VIGIA",
									  JOptionPane.INFORMATION_MESSAGE);
									  //new ImageIcon((Image)getIconImages().get(0))); 
		return true; // always true	
	}

	/**
	 * @see org.vigia.client.gui.view.View#showImage(java.awt.image.BufferedImage)
	 */
	public boolean showImage(BufferedImage frame) throws VigiaClientException {
		imPanel.setImage(frame);
		imPanel.repaint();
		return true;
	}

	/**
	 * @see org.vigia.client.gui.view.View#showLog()
	 */
	public boolean showLog() throws VigiaClientException {
		logDetails.setVisible(true);			
		return true;
	}

	
	/**
	 * @see org.vigia.client.gui.view.View#controller(org.vigia.client.gui.controller.Controller)
	 */
	public void controller(Controller ctr) {
		al = ctr;
		// Menu items
		jMI_Conn.addActionListener(ctr);
		jMI_Conn.setActionCommand(ActCmdPANEL_CONN);
		jMI_Disc.addActionListener(ctr);
		jMI_Disc.setActionCommand(ActCmdPANEL_DISC);
		jMI_Close.addActionListener(ctr);
		jMI_Close.setActionCommand(ActCmdPANEL_CLOSE);
		jMI_Config.addActionListener(ctr);
		jMI_Config.setActionCommand(ActCmdPANEL_CONFIG);
		jMI_ScrShot.addActionListener(ctr);
		jMI_ScrShot.setActionCommand(ActCmdPANEL_SCRSHOT);		
		jMI_ShowLog.addActionListener(ctr);
		jMI_ShowLog.setActionCommand(ActCmdPANEL_SHOWLOG);		
		jMI_About.addActionListener(ctr);
		jMI_About.setActionCommand(ActCmdPANEL_ABOUT);
		imPanel.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2){
					al.actionPerformed(new ActionEvent(e.getSource(),e.getID(),ActCmdPANEL_SCRSHOT));
				}
			}
		});		
	}

	/**
	 * @see org.vigia.client.gui.view.View#showAdvancedConnectionSettingsPanel()
	 */
	public void showAdvancedConnectionSettingsPanel() throws VigiaClientException {
		int temp_ip1, temp_ip2, temp_ip3, temp_ip4;
		if (connectVisible) {
			temp_ip1 = dialogConnect.getIP1();
			temp_ip2 = dialogConnect.getIP2();
			temp_ip3 = dialogConnect.getIP3();
			temp_ip4 = dialogConnect.getIP4();
		} else {
			temp_ip1 = ip1;
			temp_ip2 = ip2;
			temp_ip3 = ip3;
			temp_ip4 = ip4;			
		}			
        dialogAdvOptions = new AdvOptions(this, temp_ip1, temp_ip2, temp_ip3, temp_ip4, data_port, control_port, al, ActCmdADVOPC_OK, ActCmdADVOPC_CANCEL);
		advOptionsVisible = true;        
		dialogAdvOptions.setVisible(true);		
	}

	/**
	 * @see org.vigia.client.gui.view.View#showConnectToServerPanel()
	 */
	public void showConnectToServerPanel() throws VigiaClientException {
        dialogConnect = new Connect(this, ip1, ip2, ip3, ip4, al, ActCmdCONN_OK, ActCmdCONN_CANCEL, ActCmdCONN_ADVOPC);
		connectVisible = true;
		dialogConnect.setVisible(true);
	}

	/**
	 * @see org.vigia.client.gui.view.View#showGetScreenShotPanel()
	 */
	public void showGetScreenShotPanel() throws VigiaClientException {
        dialogScreenShot = new ScreenShot(this, target_dir, al, ActCmdSCRSHOT_OK, ActCmdSCRSHOT_CANCEL);        
		dialogScreenShot.setVisible(true);				
	}

	/**
	 * @see org.vigia.client.gui.view.View#closeAdvancedConnectionSettingsPanel()
	 */
	public void closeAdvancedConnectionSettingsPanel() throws VigiaClientException {
		dialogAdvOptions.setVisible(false);
		advOptionsVisible = false;
		ip1 = dialogAdvOptions.getIP1();
		ip2 = dialogAdvOptions.getIP2();
		ip3 = dialogAdvOptions.getIP3();
		ip4 = dialogAdvOptions.getIP4();
		data_port = dialogAdvOptions.getDataPort();
		control_port = dialogAdvOptions.getControlPort();
		if (connectVisible) {
			dialogConnect.setIP(ip1,ip2,ip3,ip4);
		}
	}

	/**
	 * @see org.vigia.client.gui.view.View#closeConnectToServerPanel()
	 */	
	public void closeConnectToServerPanel() throws VigiaClientException {
		dialogConnect.setVisible(false);
		connectVisible = false;
	}

	/**
	 * @see org.vigia.client.gui.view.View#closeGetScreenShotPanel()
	 */	
	public void closeGetScreenShotPanel() throws VigiaClientException {
		dialogScreenShot.setVisible(false);
	}
	

	/**
	 * @see org.vigia.client.gui.view.View#setStatusInfo(String status)
	 */
	public void setStatusInfo(String status) {
		jL_StatusBar.setText(status);
		
	}

	/**
	 * @see org.vigia.client.gui.view.View#appendLog(org.vigia.shared.communications.ELog, java.lang.String)
	 */
	public void appendLog(ELog log, String message) throws VigiaClientException {
		logDetails.appendLog(log, message);
		
	}	

}
