package org.vigia.client.gui.view;

import java.awt.image.BufferedImage;
import org.vigia.client.exception.VigiaClientException;
import org.vigia.client.gui.controller.Controller;
import org.vigia.shared.communications.ELog;


/**
 * The view of the application 
 * @author Ada & Toshi
 * @version 1.0 (16/11/08)
 */
public interface View {

	/**
	 * Constant for Action Command in controller. Panel: CONNECT
	 */
	public static final String ActCmdPANEL_CONN     = "PANEL_CONN";
	/**
	 * Constant for Action Command in controller. Panel: DISCONNECT  
	 */
	public static final String ActCmdPANEL_DISC     = "PANEL_DISC";
	/**
	 * Constant for Action Command in controller. Panel: CLOSE
	 */
	public static final String ActCmdPANEL_CLOSE    = "PANEL_CLOSE";
	/**
	 * Constant for Action Command in controller. Panel: SHOW CONFIG PANEL
	 */
	public static final String ActCmdPANEL_CONFIG   = "PANEL_CONFIG";
	/**
	 * Constant for Action Command in controller. Panel: SHOW SCREENSHOT
	 */
	public static final String ActCmdPANEL_SCRSHOT  = "PANEL_SCRSHOT";		
	/**
	 * Constant for Action Command in controller. Panel: SHOW LOG
	 */
	public static final String ActCmdPANEL_SHOWLOG  = "PANEL_SHOWLOG";		
	/**
	 * Constant for Action Command in controller. Panel: SHOW ABOUT
	 */
	public static final String ActCmdPANEL_ABOUT    = "PANEL_ABOUT";	
	/**
	 * Constant for Action Command in controller. Connection dialog: OK
	 */
	public static final String ActCmdCONN_OK	    = "CONN_OK";
	/**
	 * Constant for Action Command in controller. Connection dialog: CANCEL
	 */
	public static final String ActCmdCONN_CANCEL    = "CONN_CANCEL";
	/**
	 * Constant for Action Command in controller. Connection dialog: ADVANCED OPTIONS
	 */
	public static final String ActCmdCONN_ADVOPC    = "CONN_ADVOPC";
	/**
	 * Constant for Action Command in controller. Advanced Option dialog: OK
	 */
	public static final String ActCmdADVOPC_OK      = "ADVOPC_OK";
	/**
	 * Constant for Action Command in controller. Advanced Option dialog: CANCEL
	 */
	public static final String ActCmdADVOPC_CANCEL  = "ADVOPC_CANCEL";
	/**
	 * Constant for Action Command in controller. Screen Shot dialog: CANCEL
	 */
	public static final String ActCmdSCRSHOT_CANCEL = "SCRSHOT_CANCEL";
	/**
	 * Constant for Action Command in controller. Screen Shot dialog: OK
	 */
	public static final String ActCmdSCRSHOT_OK     = "SCRSHOT_OK";
	/**
	 * Shows the dialog box to connect to a server.
	 * @throws VigiaClientException error from the View.
	 */
	public void showConnectToServerPanel() throws VigiaClientException;

	/**
	 * Close the dialog box to connect to a server.
	 * @throws VigiaClientException error from the View.
	 */
	public void closeConnectToServerPanel() throws VigiaClientException;	
	
	/**
	 * Shows the dialog box to change the advanced setting of the connection.
	 * @throws VigiaClientException error from the View.
	 */
	public void showAdvancedConnectionSettingsPanel() throws VigiaClientException;
	
	/**
	 * Close the dialog box to change the advanced setting of the connection.
	 * @throws VigiaClientException error from the View.
	 */
	public void closeAdvancedConnectionSettingsPanel() throws VigiaClientException;

	/**
	 * Gets the IP address of the server from a dialog box.
	 * @return IP address.
	 * @throws VigiaClientException error from the View.
	 */
	public String getIPServer() throws VigiaClientException;
	
	/**
	 * Gets the data port from a dialog box. The data port is used to
	 * receive the images from the server.
	 * @return Data port.
	 * @throws VigiaClientException error from the View.
	 */
	public String getDataPort() throws VigiaClientException;
	
	/**
	 * Gets the control port from a dialog box. The control port is used
	 * to send commands and receive status messages.
	 * @return Control port.
	 * @throws VigiaClientException error from the View.
	 */
	public String getControlPort() throws VigiaClientException;
	
	/**
	 * Gets the desired resolution of the screen.
	 * @return Resolution in the form AxB.
	 * @throws VigiaClientException error from the View.
	 */
	public String getResolution() throws VigiaClientException;
	
	/**
	 * Shows an image in the panel. 
	 * @param frame the image to show.
	 * @return true if it was successful, false in another case.
	 * @throws VigiaClientException error from the View.
	 */
	public boolean showImage(BufferedImage frame) throws VigiaClientException;
	
	/**
	 * Gets a screenshot of the video.
	 * @return a frame.
	 * @throws VigiaClientException error from the View.
	 */
	public BufferedImage getImage() throws VigiaClientException;
	
	/**
	 * Shows the dialog box for getting a screen shot.
	 * @throws VigiaClientException error from the View.
	 */
	public void showGetScreenShotPanel() throws VigiaClientException;
	
	/**
	 * Close the dialog box for getting a screen shot.
	 * @throws VigiaClientException error from the View.
	 */
	public void closeGetScreenShotPanel() throws VigiaClientException;

	/**
	 * Gets the url where the screenshot will be saved.
	 * @return an url.
	 * @throws VigiaClientException error from the View.
	 */
	public String getScreenShotURL() throws VigiaClientException;
	
	/**
	 * Reads a log file and shows its content in a added window.
	 * @return true if it was successful, false in another case.
	 * @throws VigiaClientException error from the View.
	 */
	public boolean showLog() throws VigiaClientException;
	
	/**
	 * Shows information about the application.
	 * @return true if it was successful, false in another case.
	 * @throws VigiaClientException error from the View.
	 */
	public boolean showAboutInfo() throws VigiaClientException;
	
	/**
	 * Adds the controller ctr to all the buttons of the View
	 * @param ctr the controller.
	 */
	public void controller(Controller ctr);
	
	/**
	 * Changes the status message.
	 * @param status The status message.
	 */
	public void setStatusInfo(String status);
	
	/**
	 * Appends a log message.
	 * @param log the log's type.
	 * @param message the message.
	 * @throws VigiaClientException error from the View.
	 */
	public void appendLog(ELog log, String message) throws VigiaClientException;
}
