package org.vigia.server;

import java.awt.image.BufferedImage;
import java.io.IOException;

import org.vigia.server.communications.ClientCommunicator;
import org.vigia.server.driver.camera.CameraCommandListener;
import org.vigia.server.driver.camera.CameraDriver;
import org.vigia.server.driver.camera.Driver;
import org.vigia.shared.communications.ELog;
import org.vigia.shared.communications.FrameListener;
import org.vigia.shared.communications.LogListener;

/**
 * The server's main class.
 * @author jrios & jesus
 */
public class Server {
	/**
	 * The main function.
	 * @param args The arguments.
	 * @throws IOException Error establishing the connection.
	 */
	public static void main(String[] args) throws IOException {
		if(args.length > 1){
			System.err.println("USE: Server <port>");
			System.exit(-1);
		}
		
		int port=2000;
		try{
			if(args.length == 1)
				port=Integer.parseInt(args[0]);
		}catch(NumberFormatException nfe){
			System.err.println("USE: Server <port>");
			System.exit(-1);
		}
		
		Driver driver = new CameraDriver();
		final ClientCommunicator cc = new ClientCommunicator(port);
		
		cc.addCommandListener(new CameraCommandListener(driver));
		
		driver.addLogListener(new LogListener(){
			public void onLog(ELog log, String message) {
				try {
					cc.sendLog(log, message);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}});
		
		driver.addFrameListener(new FrameListener(){
			public void onNewFrame(BufferedImage frame) {
				try {
					cc.sendFrame(frame);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}});
	}
}
