package org.vigia.server.communications;

import java.awt.image.BufferedImage;
import java.io.IOException;

import org.vigia.server.exception.NotConnectedException;
import org.vigia.shared.communications.ELog;

/**
 * An implementation of the server communication interface.
 * @author  jrios
 */
public class ClientCommunicator implements Communicator {
	
	/**
	 * The data communicator.
	 */
	protected ControlCommunicator controlCommunicator;
	
	/**
	 * The control communicator.
	 */
	protected DataCommunicator dataCommunicator;
	
	/**
	 * Creates a communicator to listen a client.
	 * @param controlPort The control port.
	 * @throws IOException Error creating the socket.
	 */
	public ClientCommunicator(int controlPort) throws IOException {
		dataCommunicator = new DataCommunicator();
		controlCommunicator = new ControlCommunicator(controlPort);
		controlCommunicator.addCommandListener(new CommandListener(){
			public void onInit(int clientPort) {
				dataCommunicator.setClientPort(clientPort);
				dataCommunicator.setClientIp(controlCommunicator.client.getInetAddress());
			}
			public void onMove(int pan, int tilt, int zoom) {}
			public void onClose() {}
		});
	}
	
	/**
	 * @see org.vigia.server.communications.Communicator#addCommandListener(org.vigia.server.communications.CommandListener)
	 */
	public void addCommandListener(CommandListener commandListener) {
		controlCommunicator.addCommandListener(commandListener);
	}

	/**
	 * @see org.vigia.server.communications.Communicator#sendLog(org.vigia.shared.communications.ELog, java.lang.String)
	 */
	public void sendLog(ELog log, String message) throws NotConnectedException, IOException {
		controlCommunicator.sendLog(log, message);
	}

	/**
	 * @see org.vigia.server.communications.Communicator#sendFrame(java.awt.image.BufferedImage)
	 */
	public void sendFrame(BufferedImage image) throws IOException {
		dataCommunicator.sendFrame(image);
	}
}
