package org.vigia.server.communications;

import java.awt.image.BufferedImage;
import java.io.IOException;

import org.vigia.server.exception.NotConnectedException;
import org.vigia.shared.communications.ELog;

/**
 * The common interface for all the server communication implementations.
 * 
 * @author jrios
 */
public interface Communicator {
	
	/**
	 * Adds a listener to obtain control messages from the remote camera.
	 * 
	 * @param controlListener an implementation of the control listener.
	 */
	public void addCommandListener(CommandListener controlListener);
	
	/**
	 * Sends an error to the client.
	 * 
	 * @param error the kind of error.
	 * @param description an error's description.
	 * @throws IOException Error sending the message.
	 * @throws NotConnectedException There isn't any client connected.
	 */
	public void sendLog(ELog error, String description) throws NotConnectedException, IOException;
	
	/**
	 * Sends a frame to the client.
	 * 
	 * @param image the frame.
	 * @throws IOException Error sending the frame.
	 */
	public void sendFrame(BufferedImage image) throws IOException;
}
