package org.vigia.server.communications;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.SocketException;

import javax.imageio.ImageIO;

/**
 * Implements the client data communication.
 * 
 * @author jrios
 */
public class DataCommunicator {
	/**
	 * The socket to communicate with the client.
	 */
	protected DatagramSocket socket;
	/**
	 * The Client's port.
	 */
	private int clientPort;
	/**
	 * The client's Internet address.
	 */
	private InetAddress clientAddr;
	/**
	 * The next sequence number.
	 */
	protected long sequence;
	/**
	 * A buffer for the image.
	 */
	protected ByteArrayOutputStream baos;
	/**
	 * Creates a data communicator.
	 * @throws SocketException Error creating the socket.
	 */
	public DataCommunicator() throws SocketException {
		socket = new DatagramSocket();
		sequence = 0;
		baos = new ByteArrayOutputStream();
	}
	
	/**
	 * Sends a frame to the client.
	 * 
	 * @param image the frame.
	 * @throws IOException Error sending the frame.
	 */
	public void sendFrame(BufferedImage image) throws IOException {
		if(clientPort==0 || clientAddr==null) return;
		
		baos.reset();
		
		ByteArrayOutputStream seqArray = new ByteArrayOutputStream();
		ObjectOutputStream oos = new ObjectOutputStream(seqArray);
		oos.writeLong(sequence++);
		oos.flush();
		
		baos.write(seqArray.toByteArray());
		ImageIO.write(image, "jpeg", baos);
		baos.flush();

		DatagramPacket p = new DatagramPacket(baos.toByteArray(), baos.size(), clientAddr, clientPort);
		
		socket.send(p);
	}

	/**
	 * Sets the client's port.
	 * @param clientPort The Client's port.
	 */
	protected void setClientPort(int clientPort) {
		this.clientPort = clientPort;
	}

	/**
	 * Sets the client's Internet address.
	 * @param clientAddr The client's Internet address.
	 */
	public void setClientIp(InetAddress clientAddr) {
		this.clientAddr = clientAddr;
	}
}
