package org.vigia.server.driver.camera;

import org.vigia.shared.communications.FrameListener;
import org.vigia.shared.communications.LogListener;

/**
 * The camera driver interface.
 * @author jrios
 */
public interface Driver {
	
	/**
	 * Send to the camera a move command in an absolute way.
	 * 
	 * @param pan increment of pan.
	 * @param tilt increment of tilt.
	 * @param zoom increment of zoom.
	 */
	public void move(int pan, int tilt, int zoom);
	
	/**
	 * Adds a listener to obtain the frames from the camera.
	 * 
	 * @param frameListener an implementation of the data listener.
	 */
	public void addFrameListener(FrameListener frameListener);
	
	/**
	 * Adds a listener to obtain control messages from the camera.
	 * 
	 * @param logListener an implementation of the control listener.
	 */
	public void addLogListener(LogListener logListener);

	/**
	 * Starts the video streaming.
	 */
	public void start();

	/**
	 * Stops the video streaming.
	 */
	public void stop();
}
