package org.vigia.server.driver.util;

import java.awt.Graphics2D;
import java.awt.image.BufferedImage;

/**
 * This class has a set of utilities to process the images.
 * 
 * @author jrios
 */
public class ImageUtil {
	/**
	 * Crop a rectangular subimage of the image.
	 * @param i the source image.
	 * @param x the x coordinate of the point where the crop starts.
	 * @param y the x coordinate of the point where the crop starts.
	 * @param w the width of the crop rectangle.
	 * @param h the height of the crop rectangle.
	 * @return the subimage.
	 */
	public static BufferedImage crop(BufferedImage i, int x, int y, int w, int h) {
		// Advice: The returned BufferedImage shares the same data array as the original image.
		if(x<0) x=0;
		if(y<0) y=0;
		if(x+w>i.getWidth()) x-=x+w-i.getWidth();
		if(y+h>i.getHeight()) y-=y+h-i.getHeight();

		return i.getSubimage(x, y, w, h);		
	}
	
	/**
	 * Scales an image to a specific resolution.
	 * @param i the source image.
	 * @param w the width of the result image.
	 * @param h the height of the result image.
	 * @return the scaled image.
	 */
	public static BufferedImage scale(BufferedImage i, int w, int h) {
		BufferedImage res = new BufferedImage(w, h, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2 = res.createGraphics();
        g2.drawImage(i, 0, 0, w, h, null);
        g2.dispose();
		return res;
	}
	
	/**
	 * Changes the perspective of an image.
	 * @param i the source image.
	 * @param x the x coordinate of the viewer.
	 * @param y the y coordinate of the viewer.
	 * @param z the z coordinate of the viewer.
	 * @return the projection of the image.
	 */
	public static BufferedImage perspective(BufferedImage i, int x, int y, int z) {
		//TODO implementation
		return i;
	}
}
