package org.vigia.test;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.text.DecimalFormat;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.vigia.server.driver.camera.JMFCapture;

/**
 * @author Jesus
 *
 */
@SuppressWarnings("serial")
public class CaptureFramesTest extends JFrame{

	/**
	 * JPicsPanel object for draw the frames
	 */
	private JPicsPanel pp;
	
	/**
	 * CaptureFramesTest constructor
	 */
	public CaptureFramesTest()
	{
	 super( "Show JMF Capture Pics" );
	 Container c = getContentPane();
	 c.setLayout( new BorderLayout() );   

	 pp = new JPicsPanel(); // the sequence of snaps appear here
	 c.add( pp, "Center");

	 addWindowListener( new WindowAdapter() 
	 { public void windowClosing(WindowEvent e)
	   { pp.closeDown();    // stop snapping pics
	     System.exit(0);
	   }
	 });
	 
	 pack();  
	 setResizable(false);
	 setVisible(true);
	}
	
	/**
	 * @author Jesus
	 *
	 */
	class JPicsPanel extends JPanel implements Runnable
	{
		/**
		 * 
		 */
		private static final int SIZEX = 640;  // x dimension of panel
		/**
		 * 
		 */
		private static final int SIZEY = 480;  // y dimension of panel
		/**
		 * 
		 */
		private static final int DELAY = 30;  // ms  (25 FPS; could be 25-35ms)
	
	
		/**
		 * 
		 */
		private BufferedImage image = null;
		/**
		 * 
		 */
		private JMFCapture camera; 
		/**
		 * 
		 */
		private boolean running;
	
		// used for the average ms snap time info
		/**
		 * 
		 */
		private int imageCount = 0;
		/**
		 * 
		 */
		private long totalTime = 0;
		/**
		 * 
		 */
		private DecimalFormat df;
		/**
		 * 
		 */
		private Font msgFont;
	
	
		/**
		 * 
		 */
		public JPicsPanel()
		{
		 setBackground(Color.white);
		 Dimension d = new Dimension(SIZEX, SIZEY);
		 setMinimumSize(d);
		 setPreferredSize(d);
	
		 df = new DecimalFormat("0.#");  // 1 dp
		 msgFont = new Font("SansSerif", Font.BOLD, 18);
	
		 new Thread(this).start();   // start updating the panel's image
		}
	
		
		/**
		 * @see java.lang.Runnable#run()
		 */
		public void run(){
		 camera = new JMFCapture();
	
		 long duration;
		 BufferedImage im = null;
		 running = true;
	
		 while (running) {
			  long startTime = System.currentTimeMillis();
			  
		   im = camera.grabImage();  // take a snap
		   duration = System.currentTimeMillis() - startTime;
	
		   if (im == null)
		     System.out.println("Problem loading image " + (imageCount+1));
		   else {
		     image = im;   // only update image if im contains something
		     imageCount++;
		     totalTime += duration;
		     repaint();
		   }
		   
		   if (duration < DELAY) {
		     try {
		       Thread.sleep(DELAY-duration);  // wait until DELAY time has passed
		     } 
		     catch (Exception ex) {}
		   }
		 }
		 
		 camera.close();    // close down the camera
		} 
	
	
		/**
		 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
		 */
		public void paintComponent(Graphics g){ 
		 super.paintComponent(g);
		 g.drawImage(image, 0, 0, this);   // draw the snap
	
		 // write stats
		 g.setColor(Color.blue);
		 g.setFont(msgFont);
		 if (imageCount > 0) {
		   double avgGrabTime = (double) totalTime / imageCount;
			  g.drawString("Pic " + imageCount + "  " +
		                df.format(avgGrabTime) + " ms", 
		                5, SIZEY-10);  // bottom left
		 }
		 else  // no image yet
			  g.drawString("Loading...", 5, SIZEY-10);
		}
	
	
		/**
		 * 
		 */
		public void closeDown(){ 
		 running = false;
		 while (!camera.isClosed()) {
		   try {
		     Thread.sleep(DELAY);
		   } 
		   catch (Exception ex) {}
		 }
		}
	}
	
	
	/**
	 * @param args arguments
	 */
	public static void main(String[] args) {
		new CaptureFramesTest();
	}
	
	
}
